﻿using System;
using System.IO;
using System.Collections;
using CitrixRADESDK;

namespace UpgradeProfiles
{
    class UpgradeProfiles
    {
        public static String AppHubLocation = "";
        public static bool CreateVHD = false;

        //UgradeProfiles
        String ProfilePath;
        String CurrentDirectory;

        public UpgradeProfiles()
        {
            ProfilePath = "";            
            CurrentDirectory = "";
        }

        /// <summary>
        /// Takes path of the AppHub on the SMB share on Profiler machine, which has Profiler SDK registered. Path should be like \\NAS\share\Apphub. 
        /// </summary>
        /// <param name="args">Takes share location of the AppHub like \\NAS\share\Apphub [/vhd]</param>
        static void Main(string[] args)
        {
            if (args.Length < 1) Usage();
            if (args[0].Length < 1 || !Directory.Exists(args[0])) Usage();

            AppHubLocation = args[0];

            if (args.Length > 1)
            {
                if (args[1].Equals("/vhd", StringComparison.CurrentCultureIgnoreCase))
                    CreateVHD = true;
            }
            Console.WriteLine("UpgradeProfiles Tool will scan the entire AppHub and upgrade Targets of " + Environment.OSVersion + ", to Current Profiler Version... \n");
            UpgradeProfilesMain();
        }


        static void UpgradeProfilesMain()
        {
            DirectoryInfo curDirInfo = new DirectoryInfo(AppHubLocation);
            DirectoryInfo[] AllSubDirs = curDirInfo.GetDirectories("*", SearchOption.AllDirectories);

            foreach (DirectoryInfo subDir in AllSubDirs)
            {
                UpgradeProfiles Profile = new UpgradeProfiles();
                Profile.CurrentDirectory = subDir.FullName;
                
                if (Profile.IfValidProfile())
                {
                    try
                    {
                        Profile.EditTarget();
                    }
                    catch (Exception)
                    {
                        Console.WriteLine("\nProfiler SDK is not registered or Profiler is not installed.");                        
                        Environment.Exit(-1);
                    }
                }
            }
        }

        /// <summary>
        /// Updates Profiles
        /// </summary>
        /// <param name="ProfilePath"></param>
        void EditTarget()
        {
            try
            {
                String mBuildLocation;
                mBuildLocation = "C:\\CTXPackager";
                CRADEPackage CurrentProfile = new CRADEPackage();

                Console.WriteLine("\nTrying to Upgrade Profile:" + ProfilePath + " ...");

                CurrentProfile.Open(ProfilePath);
                CurrentProfile.Edit(mBuildLocation);

                CRADETarget CurrentTarget = CurrentProfile.FindTargetForCurrentOS();
                if (CurrentTarget.MinimalTarget != 1)
                {
                    //Target App
                    CurrentTarget.Edit(CurrentProfile.TgtBuildLocation);

                    //If the profiler Version is above 6.5 then admin can create VHD for all the target to optimise Streaming for
                    //for Pooled XenDesktop, by setting /vhd to 1
                    if (CreateVHD)
                    {
                        //uncomment the next line if the profiler version is v6.5 or later
                        //CurrentTarget.VirtualHardDisk = 1;
                    }

                        CurrentTarget.RunDiscovery();
                    VerifySandboxProc(CurrentTarget);
                }
                CurrentProfile.Save();
            }
            catch (Exception e)
            {
                Console.WriteLine("Target Upgrade for Profile:" + ProfilePath + " Failed, Exception:" + e.Message);
                return;
            }
            Console.WriteLine("Profile:" + ProfilePath + " Upgraded successfully.");
        }

        private bool IfValidProfile()
        {
            DirectoryInfo dirInfo = new DirectoryInfo(CurrentDirectory);
            //if .Profile Exists with DirectoryName/Directory.Profile
            if (File.Exists(CurrentDirectory + "\\" + dirInfo.Name + ".profile"))
            {
                ProfilePath = CurrentDirectory + "\\" + dirInfo.Name + ".profile";
                return true;
            }
            else return false;
        }

        // Checks if any process of installer is still running.
        private static void VerifySandboxProc(CRADETarget CurrentTarget)
        {
            object[] running_procs = null;
            running_procs = CurrentTarget.EnumSandboxProcesses() as object[];
            if (running_procs != null && running_procs.Length > 0)
            {
                CurrentTarget.KillSandboxProcesses(running_procs);
            }
        }

        static void Usage()
        {
            Console.WriteLine("Usage: UpgradeProfiles <AppHubPath> [/vhd] ");
            Console.WriteLine("");
            Console.WriteLine("         <AppHubPath>: Location of AppHub where profiles are shared");
            Console.WriteLine("         /vhd        : Optional for creation of VHD files for Pooled XD");
            Environment.Exit(-1);
        }
    }
}